#!/usr/bin/perl

#-----------------------------------------------------------------------------------------------------------------------------------------------------------
#
# Copyright © 2008 Novell, Inc.  All Rights Reserved.
#
# THIS WORK IS SUBJECT TO U.S. AND INTERNATIONAL COPYRIGHT LAWS AND TREATIES.  IT MAY NOT BE USED, COPIED, DISTRIBUTED, DISCLOSED, ADAPTED,
# PERFORMED, DISPLAYED, COLLECTED, COMPILED, OR LINKED WITHOUT NOVELL'S PRIOR WRITTEN CONSENT.  USE OR EXPLOITATION OF THIS WORK WITHOUT
# AUTHORIZATION COULD SUBJECT THE PERPETRATOR TO CRIMINAL AND CIVIL LIABILITY.
#
# NOVELL PROVIDES THE WORK "AS IS," WITHOUT ANY EXPRESS OR IMPLIED WARRANTY, INCLUDING WITHOUT THE IMPLIED WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE, AND NON-INFRINGEMENT. NOVELL, THE AUTHORS OF THE WORK, AND THE OWNERS OF COPYRIGHT IN THE WORK ARE NOT
# LIABLE FOR ANY CLAIM, DAMAGES, OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT, OR OTHERWISE, ARISING FROM, OUT OF, OR IN
# CONNECTION WITH THE WORK OR THE USE OR OTHER DEALINGS IN THE WORK.
#
#-----------------------------------------------------------------------------------------------------------------------------------------------------------



use strict;
use Getopt::Long;
use Pod::Usage;
use File::Copy;


# *** NOTE ***
# Additional option to migafp : -u, --username=USERNAME        The user name to use on the source server
# This is used only by the GUI. Hence this is undocumented.

####	POD begin	####
#FIXME::Need to check how the = is to be taken
=head1 NAME
	
migafp.pl - Migrates the context file of NetWare AFP to AFP on Linux

=head1 SYNOPSIS

 migafp.pl -s SOURCE_IP -u USERNAME [OPTIONS]
	OR
 migafp.pl --source-server=SOURCE_IP --username=USERNAME [OPTIONS]

 where SOURCE_IP is the IP address of the source server for the migration operation

=head1 OPTIONS
				
	
 -p, --password=password 	The password of the user on the source server.

 -m, --mode=mode 		The type of migration to be done.
				
 -c, --contexts=contexts 	The contexts on the source server.
 				
 -h, --help, --usage		Prints a brief help message and exits
     --man		   	Prints the manual page and exits

=head1 DESCRIPTION

This program reads the NetWare AFP context file and writes back the mapping to 
the /etc/opt/novell/afptcpd/afpdircxt.conf file of AFP on Linux

=cut

####	POD end		####
use Getopt::Std;
# Globals
my ($srchost, $user, $password) = ("", "", "");
my $basedir = "/mnt/migration";
my $mntdir = "$basedir/afp";
my $mntdirpath = "$mntdir/SYS";
my $srcfile = "";
my $destfile = "/etc/opt/novell/afptcpd/afpdircxt.conf";
my $tempfile= "/tmp/afptemp.conf";
my $mode= "";
my $cxts="";
my @nw_options;

my $srcldapcred;
my $srcldappwd;
my $srcldapport;
my $srcssl;

my $destldapcred;
my $destldappwd;
my $destldapport;
my $destssl;
my %args;
# Logs error messages to STDERR and /var/log/migafp.log
# logmsg (custom_error_message, error_message) 
sub logmsg ($;$;$)
{
	my $error_message = $_[1];
	if ( $_[0] ne "")
	{
		print STDERR "$_[2]:$_[0]\n";
	}

	print LOGFILE localtime() . " $_[0]";
	if ( $error_message ne "" )
	{
		#print STDERR "$_[0]\n";
		print LOGFILE " : $error_message";
	}
	print LOGFILE "\n";
}

sub mount()
{
	# Mount the NetWare volume
	if ( ! -d $mntdir )
	{
		if ( ! -d $basedir )
		{
			if ( !mkdir ($basedir, 0755) )
			{
				logmsg "Could not create the directory $basedir", $!, "Fatal";
		        	exit 2;
			}
		}
		if ( !mkdir ($mntdir, 0755) )
		{
			logmsg "Could not create the directory $mntdir", $!, "Fatal";
			exit 2;
		}
	}

	my $result = `/usr/bin/nwmap -s "$srchost" -d $mntdirpath -v SYS -u "$user" -p "$password" 2>&1 1>/dev/null`;
	#logmsg "Result = $result";
	if ( $result ne "" )
	{
		logmsg "Could not mount $srchost. $result", $?, "Fatal";
		exit 2;
	}
	else
	{
	 	logmsg "Mounted $srchost at $mntdir","","Information";
	}
}


# Unmount ncp volume and remove temporary mount point
sub unmount
{
	`/usr/bin/nwlogout -a`;
	logmsg "Unmounted $mntdir","","Information";
	rmdir($mntdir);
}


sub gethostip()
{
  use Socket 'inet_ntoa';
  use Sys::Hostname 'hostname';
  my $addr = inet_ntoa(scalar gethostbyname(hostname() || 'localhost'));
  return $addr;
}


sub validateinputs()
{
	#    if ($srchost eq "" )
	#{
	# if($mode ne "" && $mode ne "5"){
	#	pod2usage (-message => "\n$0 : Source server IP address not specified\n",
	#	   -exitval => 2,
	#	   -verbose => 1);
	# }else{
	#	$srchost=gethostip();
	#   }
	#}else {
	
	if ( $password eq "" )
	{
	        system ( "/opt/novell/migration/sbin/migcred -i $srchost -n $user" );
	        chomp ( $user = `/opt/novell/migration/sbin/migcred -i $srchost -n -r 2>/dev/null` );
	        $user =~ s/[\s]*nds_dn[\s]*:[\s]*//;
	        $user=~ s/[\s]*$//;
	        chomp ( $password = `/opt/novell/migration/sbin/migcred -i $srchost -n -w -r 2>/dev/null` );
	        $password =~ s/[\s]*password[\s]*:[\s]*//;
	        $password =~ s/[\s]*$//;
	}

#	unless ($srchost =~ /^\d+\.\d+\.\d+\.\d+/)
#	{
#		pod2usage (-message => "\n$0 : invalid source server address '$srchost'\n",
#	        -exitval => 2,
#	        -verbose => 1);
#	}
#	}
	
#      if($user eq ""){

#		pod2usage (-message => "\n$0 : Username not specified\n",
#		   -exitval => 2,
#		   -verbose => 1);
#	}
	#FIXME neednt check for username i guess
}

sub getpassword()
 {
 # read source credentials
 $password = $ENV{'PASSWORD'};

 # This means that the environment variable was not set.
 # Hence we fall back to 'migcred' for completing authentication.
 if ( $password eq "" )
  {
	#system ( "/opt/novell/migration/sbin/migcred -i $srchost -n $user" );

	#chomp ( $user = `/opt/novell/migration/sbin/migcred -i $srchost -n -r 2>/dev/null` );
        #$user =~ s/[\s]*nds_dn[\s]*:[\s]*//;
       # $user=~ s/[\s]*$//;

        $password = `/opt/novell/migration/sbin/migcred -i $srchost -n -w -r 2>/dev/null` ;
	$password =~ s/[\s]*password[\s]*:[\s]*//;
	$password =~ s/[\s]*$//;
  }

}
sub readsrcfile()
{
	# Read the ctxs.cfg configuration file & create the mapping
	my $name;
	if (!open (NWAFP_HANDLE, "/mnt/migration/afp/SYS/etc/ctxs.cfg"))
	{
  		if (!open (NWAFP_HANDLE, "/mnt/migration/afp/SYS/etc/CTXS.CFG"))
  		{
			logmsg "No configuration file found", "No configuration file found", "Fatal";
			unmount ();
			exit 2;
  		}
	}

	my $newline;
	my $treename;
        $treename = `cat /etc/opt/novell/eDirectory/conf/nds.conf | grep "n4u.base.tree-name"`;
	if($treename =~ /^n4u.base.tree-name\s*\=\s*(.*)/i)
	{
        	$treename = $1;
	}
	else
	{
			logmsg "Unable to obtain the server tree name", "Unable to obtain the server tree name", "Error";
			unmount ();
			exit 2;
		
	}
  	foreach(<NWAFP_HANDLE>)
  	{
	    	chomp;

		if ( ! /^[#\n\r]/ )			# Unless it is a comment or a newline
		{
			$newline=$_;
			if (defined ($newline))
			{
		  		$newline =~ s/^\s+//;
		  		$newline =~ s/\s+$//;
			}
			if (($newline =~ /=/) && !($newline =~ /\\=/))
			{
				push (@nw_options, $newline);
			}
			else
			{
				$newline = `/opt/novell/migration/sbin/afpcontext $treename $newline`;
				if($? != 0)
				{
				   logmsg "afpcontext failed to obtain typed context name for: $newline", "afpcontext failed to obtain typed context name: $newline", "Error";
				}
				push (@nw_options, $newline);
			}
		}
   	}
   	close (NWAFP_HANDLE);
}

sub readdestfile
{
	# check if /etc/opt/novell/afptcpd/afpdircxt.conf already exists

	if (-e "$destfile")
	{
	  # copy /etc/opt/novell/afptcpd/afpdircxt.conf to /etc/opt/novell/afptcpd/afpdircxt.conf.old
 
	  logmsg "Copying $destfile file to $destfile.old","","Information";

	  if (!copy ("$destfile", "$destfile.old"))
	  {
	    logmsg "Error copying the $destfile file. Only root user has access to copy this file","","Fatal";
	    unmount ();
	    exit (1);
	  }
	}
	else
	{
	    logmsg "$destfile file does not exist. Ensure AFP is installed on the target server","","Fatal";
	    unmount ();
	    exit (1);
	}

	# open the /etc/opt/novell/afptcpd/afpdircxt.conf file of the Linux server in write mode

	logmsg "Copied old $destfile file to $destfile.old","","Information";
       # logmsg "Opening $destfile file in read mode\n";

	#if (!open (AFP_IN_HANDLE, ">$destfile"))
	#{
	#  print LOGFILE " Error opening the $destfile file. Only root user has write access to this file.\n";
	#  close (AFP_IN_HANDLE);
	#  unmount ();
	#  print STDERR "Error opening the $destfile file\nPlease see the logfile for more details.\n";
	#  exit (1);
	#}

}
sub checkExist($)
{

 if ( !open (AFP_IN_CHECKHANDLE,"$destfile") )
 {
	logmsg "Could not open $destfile for reading. Ensure AFP server is installed on the target", $!,"Fatal";
	unmount ();
	exit 2;
 }	

 my $line;
 my $checkVar=$_[0];
# $checkVar=~ s/\s//g;
 
 while ($line = <AFP_IN_CHECKHANDLE>)
 {
	if ( $line =~ /^$checkVar/i )
	{
		logmsg "entry $_[0] is duplicate","","Information";
		return "exists";
	}
 }
  close(AFP_IN_CHECKHANDLE);
  return "0";
 
}

sub writetofile($)
{
 
   if ( !open (AFP_WRITE_HANDLE,">>$destfile") )
    {
	logmsg "Could not open $destfile for writing", $!,"Fatal";
	logmsg "Trying to create the file", $destfile,"Information";
	if ( !open (AFP_WRITE_HANDLE, ">$destfile") )
	{
		logmsg "Could not create $destfile\n", $!,"Fatal";
		unmount ();
		exit 2;
	}
	logmsg "File created", $destfile,"Information";
    }

    print AFP_WRITE_HANDLE "$_[0]\n";

    close(AFP_WRITE_HANDLE);
}


%args=();
#getopt("m:s:u:w:",\%args);
#getopt("s:u:w:",\%args);
getopt("s:u:",\%args);

#$mode = $args{m};
$user = $args{u};
$srchost = $args{s};
#$password  = $args{w};
$password  = $ENV{'PASSWORD'};



if ($mode eq "1" ){
	# serveridswap

}elsif ($mode eq "2" ){
	# different tree 

}elsif ($mode eq "3" ){
	# different tree different context	

}elsif ($mode eq "4" ){
	#consolidation

}else{
	#assume its same tree migration
 }


validateinputs();




if ( !open (LOGFILE, ">/var/log/migafp.log") )
{
	logmsg "Unable to create log file - /var/log/migafp.log","","Fatal";
	exit 2;
}

#if($password eq ""){
#
#	getpassword();
#}

mount();

readsrcfile();

readdestfile();

if ( !open (AFP_TEMP_HANDLE, ">$tempfile") )
{
	logmsg "Could not open $tempfile for writing", $!,"Fatal";
	unmount();
	exit 2;
}

logmsg "Temporary File created", $tempfile,"Information";

for my $p (0 .. $#nw_options)
    {
	if(!checkExist($nw_options[$p]))
	{
        	writetofile($nw_options[$p]);
        	print AFP_TEMP_HANDLE "$nw_options[$p]\n";
	}
    }
# rename the file back
close (AFP_TEMP_HANDLE);

logmsg "$destfile saved with the migrated configuration parameters","","Information";

close (LOGFILE);

unmount ();

exit 0;
