#!/usr/bin/perl

#-----------------------------------------------------------------------------------------------------------------------------------------------------------
#
# Copyright © 2007 Novell, Inc.  All Rights Reserved.
#
# THIS WORK IS SUBJECT TO U.S. AND INTERNATIONAL COPYRIGHT LAWS AND TREATIES.  IT MAY NOT BE USED, COPIED, DISTRIBUTED, DISCLOSED, ADAPTED,
# PERFORMED, DISPLAYED, COLLECTED, COMPILED, OR LINKED WITHOUT NOVELL'S PRIOR WRITTEN CONSENT.  USE OR EXPLOITATION OF THIS WORK WITHOUT
# AUTHORIZATION COULD SUBJECT THE PERPETRATOR TO CRIMINAL AND CIVIL LIABILITY.
#
# NOVELL PROVIDES THE WORK "AS IS," WITHOUT ANY EXPRESS OR IMPLIED WARRANTY, INCLUDING WITHOUT THE IMPLIED WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE, AND NON-INFRINGEMENT. NOVELL, THE AUTHORS OF THE WORK, AND THE OWNERS OF COPYRIGHT IN THE WORK ARE NOT
# LIABLE FOR ANY CLAIM, DAMAGES, OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT, OR OTHERWISE, ARISING FROM, OUT OF, OR IN
# CONNECTION WITH THE WORK OR THE USE OR OTHER DEALINGS IN THE WORK.
#
#-----------------------------------------------------------------------------------------------------------------------------------------------------------



use strict;
use Getopt::Long;
use Pod::Usage;
use Getopt::Std;


# Globals

my $srcfile = "/tmp/afptemp.conf";

my $source;

my $port;
my $ssl="";
my $ldap;
my $binddn;
my $bindpasswd;
my $mesg;
my $locatorbase;
my $defaultpwdpolicyname="AFP-CIFS Migration Policy";
my @cxts;
my @policiestobemodified;
my $count;
my $proxyuser;
my $pwdpolicyfile;
my %args;
my @modifiedPolicies;
my %modified_policy_hash = ();
# Logs error messages to STDERR and /var/log/migafp.log
# logmsg (custom_error_message, error_message) 

open (SERVERFILE,">/tmp/afpuserpwdpolicy.ldif")          
or &fileerror($!);

sub logmsg ($;$;$)
{
	my $error_message = $_[1];
	if ( $_[0] ne "")
	{
		print STDERR "$_[2]:$_[0]\n";
	}

	print LOGFILE localtime() . " $_[0]";
	if ( $error_message ne "" )
	{
		#print STDERR "$_[0]\n";
		print LOGFILE " : $error_message";
	}
	print LOGFILE "\n";
}


sub gethostip()
{
  use Socket 'inet_ntoa';
  use Sys::Hostname 'hostname';
  my $addr = inet_ntoa(scalar gethostbyname(hostname() || 'localhost'));
  return $addr;
}

sub createpwdpolicy(){

	print SERVERFILE "dn: cn=$defaultpwdpolicyname,cn=Password Policies,cn=Security\nnsimPwdRuleEnforcement: FALSE\nnsimForgottenAction::  PEZvcmdvdHRlblBhc3N3b3JkPjxFbmFibGVkPmZhbHNlPC9FbmFibGVk
 PjxTZXF1ZW5jZT48QXV0aGVudGljYXRpb24+PCFbQ0RBVEFbXV0+PC9BdXRoZW50aWNhdGlvbj48Q
 WN0aW9uPjwvQWN0aW9uPjwvU2VxdWVuY2U+PC9Gb3Jnb3R0ZW5QYXNzd29yZD4=\n";
	print SERVERFILE "nsimForgottenLoginConfig: TRUE\n";
	print SERVERFILE "nspmCaseSensitive: TRUE\n";
	print SERVERFILE "nspmSpecialAsLastCharacter: TRUE\n";
	print SERVERFILE "nspmSpecialAsFirstCharacter: TRUE\n";
	print SERVERFILE "nspmSpecialCharactersAllowed: TRUE\n";
	print SERVERFILE "nspmNumericAsLastCharacter: TRUE\n";
	print SERVERFILE "nspmNumericAsFirstCharacter: TRUE\n";
	print SERVERFILE "nspmNumericCharactersAllowed: TRUE\n";
	print SERVERFILE "nspmConfigurationOptions: 852\n";
	print SERVERFILE "passwordUniqueRequired: FALSE\n";
	print SERVERFILE "passwordAllowChange: TRUE\n";
	print SERVERFILE "nspmPasswordACL: 2#entry#$proxyuser#nspmPassword\n";
	print SERVERFILE "objectClass: nspmPasswordPolicy\n";
	print SERVERFILE "objectClass: Top\n";
	print SERVERFILE "cn: $defaultpwdpolicyname\n";
	print SERVERFILE "\n";
}


# Returns the mapped result.
sub establishconnection()
{
	 logmsg "Source being used>$source","","Information";
use Switch;
	if($ssl)
	{
		use Net::LDAPS;
		use Net::LDAP::Util qw(ldap_error_text
                         ldap_error_name
                         ldap_error_desc
                        );                                              #SSL bind 

		$ldap = Net::LDAPS->new($source,
                          port => $port);
		if ($@ ne  "")
   		{
		    logmsg "$@ This may be due to incorrect server name or port number while using ssl",$!, "Fatal";
		    print stderr "An ldap error occurred. Please see the log file for more details.\n";
                    exit(2);
   		} 
	}
	else
	{
		use Net::LDAP;
		use Net::LDAP::Util qw(ldap_error_text
                         ldap_error_name
                         ldap_error_desc
                        );                                             # Non-SSL bind
		$ldap = Net::LDAP->new($source,
                          port => $port);
  		if ($@ ne  "")
   		{
    			logmsg "$@ This may be due to incorrect server name or port number",$!, "Fatal";
    			exit(2);
   		}
	}

	$mesg = $ldap->bind ($binddn, password => $bindpasswd);
 	my $check=ldap_error_name($mesg);
	if (ldap_error_name($mesg) ne "LDAP_SUCCESS")
 	{
		 logmsg "Authentication to ldap failed. Error: $check",$!, "Fatal";
		 print stderr "An ldap error occurred. Please see the log file for more details.\n";
		 print stderr ldap_error_name($mesg);
		 print stderr ldap_error_text($mesg);
		 exit(2);
 	}
	else
	{
  		logmsg "Successfully established ldap connection","","Information";
	}
}


sub createPasswordPolicyattachingldif($) #input would be the object to which we have to attach the policy
{
	print SERVERFILE "dn: $_[0]\nchangetype: modify\nadd: nspmPasswordPolicyDN\n";
	print SERVERFILE "nspmPasswordPolicyDN: cn=$defaultpwdpolicyname,cn=Password Policies,cn=Security\n";
	print SERVERFILE "\n";

	print SERVERFILE "dn: cn=$defaultpwdpolicyname,cn=Password Policies,cn=Security\nchangetype: modify\nadd: nsimAssignments\n";
	print SERVERFILE "nsimAssignments: $_[0]\n";
	print SERVERFILE "\n";
}


sub modifyPasswordpolicyattachingldif($)
{
	print SERVERFILE "dn: $_[0]\nchangetype: modify\nreplace: nspmPasswordPolicyDN\n";
	print SERVERFILE "nspmPasswordPolicyDN: cn=$defaultpwdpolicyname,cn=Password Policies,cn=Security\n";
	print SERVERFILE "\n";

	print SERVERFILE "dn: cn=$defaultpwdpolicyname,cn=Password Policies,cn=Security\nchangetype: modify\nadd: nsimAssignments\n";
	print SERVERFILE "nsimAssignments: $_[0]\n";
	print SERVERFILE "\n";

}

sub attachProxyUsertodefaultPolicy()
{
	print SERVERFILE "dn: $proxyuser\nchangetype: modify\nreplace: nspmPasswordPolicyDN\n";
	print SERVERFILE "nspmPasswordPolicyDN: cn=$defaultpwdpolicyname,cn=Password Policies,cn=Security\n";
	print SERVERFILE "\n";

	print SERVERFILE "dn: cn=$defaultpwdpolicyname,cn=Password Policies,cn=Security\nchangetype: modify\nadd: nsimAssignments\n";
	print SERVERFILE "nsimAssignments: $proxyuser\n";
	print SERVERFILE "\n";
}

sub checkifdefaultpolicyexists()
{
	$locatorbase="cn=$defaultpwdpolicyname,cn=Password Policies,cn=Security"; #the default policy if it exists should exist in the Password Policies Container under Security Container
	$mesg = $ldap->search(
	# perform a search
                        base => $locatorbase,scope=>"base",  filter => "(objectclass=nspmPasswordPolicy)");

	if($mesg->count eq 0) #this means that policy does not exists.....
	{
		return "0";
	}
	else
	{

		return "1";
	}	

}

#keeping for time being..May have to remove later.
sub getallusers($){

my $cxt=$_[0];
my $index;
$index=rindex($cxt,"=");
$cxt=substr($cxt,$index-1);

$mesg = $ldap->search (
# perform a search
                        base => "o=novell", filter => "(objectclass=User)" ,attrs=> ['cn', 'nspmPasswordPolicyDN']);
my @users=$mesg->entries;
my $numofusers = $mesg->count;

my @attrMap;

if ($numofusers ne 0){
for my $p (0 .. $#users)
    {
	if($users[$p]->get_value("nspmPasswordPolicyDN") eq ""){
		createPasswordattachingldif($users[$p]->dn);
	}else{
		#should check if the policy is password enabled..if not do it.
	}
   }
}
}

#keeping for time being..May have to remove later.
sub universalcheckuser($)
{ 
#	logmsg "in universalcheck the context coming is>$_[0]";
	getallusers($_[0]);
}


# this method will ensure that the pwd policy allows user to retrieve pwd,is Universal Password enabled and will attach the proxy user to this password policy	
sub checkifpolicyispwdenabled($)
{	
	
	my $policy=$_[0];
	$locatorbase="$policy";
	logmsg "Querying for policy $policy","","Information";
	$mesg = $ldap->search(
	# perform a search
                        base => $locatorbase,scope=>"base", filter => "(objectclass=nspmPasswordPolicy)");

	if($mesg->count ne 0) #this means that policy exists.....
	{
		logmsg "Querying for $policy successful","","Information";
		my $universalVal=256;	#decimal equivalent of 0x100 .For more information refer to /opt/novell/sch/nspm.sch
		my $checkentry=$mesg->entry;
		my $nspmconfigvalue=$mesg->entry->get_value("nspmConfigurationOptions");
		my $nspmPasswordACL=$mesg->entry->get_value("nspmPasswordACL");
		my $res= $nspmconfigvalue & $universalVal;
	
		if($res eq 0) #Not UP enabled
		{
			#need to modify the pwd policy to make it universal enabled
			$nspmconfigvalue= $nspmconfigvalue | $universalVal;
			print SERVERFILE "dn: $policy\nchangetype: modify\nreplace: nspmConfigurationOptions\n";
			print SERVERFILE "nspmConfigurationOptions: $nspmconfigvalue\n";
			print SERVERFILE "\n";
			logmsg "Modifying the policy $policy to make it UP enabled","","Information";
		}

		#need to check if it allows the user to retrieve password
		$universalVal=16;       #decimal equivalent of Ox10 .For more information refer to /opt/novell/sch/nspm.sch
		$res= $nspmconfigvalue & $universalVal; 
		if($res eq 0) #Allow password retrieval by self is not enabled
		{
			#need to modify the pwd policy to make it allow user to retrieve password
			$nspmconfigvalue= $nspmconfigvalue | $universalVal;
			print SERVERFILE "dn: $policy\nchangetype: modify\nreplace: nspmConfigurationOptions\n";
			print SERVERFILE "nspmConfigurationOptions: $nspmconfigvalue\n";
			print SERVERFILE "\n";		
			logmsg "Modifying the policy $policy to allow user to retrieve password","","Information";
		}
		
		#need to add proxy user to this policy to allow to retrieve password
		#need to get the proxy user and assign it here...
		
		if($nspmPasswordACL ne ""){
			my $check =$nspmPasswordACL =~ m/$proxyuser/i;
			if(!$check){
				push (@modifiedPolicies,$policy); #to avoid duplicate add operations on the same policy.
				print SERVERFILE "dn: $policy\nchangetype: modify\nadd: nspmPasswordACL\n";
				print SERVERFILE "nspmPasswordACL: 2#entry#$proxyuser#nspmPassword\n";
				print SERVERFILE "\n";		
				logmsg "Modifying the policy $policy to allow $proxyuser to retrieve password","","Information";
			}
		}else {
			push (@modifiedPolicies,$policy); #to avoid duplicate add operations on the same policy.
			print SERVERFILE "dn: $policy\nchangetype: modify\nadd: nspmPasswordACL\n";
			print SERVERFILE "nspmPasswordACL: 2#entry#$proxyuser#nspmPassword\n";
			print SERVERFILE "\n";		
			logmsg "Modifying the policy $policy to allow $proxyuser to retrieve password","","Information";
		}
		
	}
	else{

	# The pwd policy is not migrated...i think this case will never arise...still...i'll attach the context to my default pwd policy
		return "0";
	}

	return "1";
}

sub modifyPolicies()
{
# open the file and read the policies into an array
	if ( !open (AFP_POLICY_HANDLE,"<$pwdpolicyfile") )
	{
		logmsg "Could not open $pwdpolicyfile", $!,"Fatal";
		return;
	}
	my $newline;

	foreach(<AFP_POLICY_HANDLE>)
	{    chomp;

		if ( ! /^[#\n\r]/ )
		{
			$newline=$_;
			if (defined ($newline))
			{
			  $newline =~ s/^\s+//;
			  $newline =~ s/\s+$//;
			}
			if(!defined($modified_policy_hash{$newline})){
				push (@policiestobemodified, $newline);
			}
			else {
				logmsg "Policy $newline has already been modified!","","Information";	
			}
        	}
	}
	close (AFP_POLICY_HANDLE);
	logmsg "The policymap is @policiestobemodified","","Information";
	my $policycount=@policiestobemodified;
	if($policycount eq 0){
		logmsg "Either no password policies were present in the input file $pwdpolicyfile or no modification required for the password policies!","","Information";
		return "1";
	}
	for my $j (0 .. $#policiestobemodified)
	{
		checkifpolicyispwdenabled($policiestobemodified[$j]);		
	}
	return;
}


sub bindcontexttopolicy($)
{
	my $context=$_[0];
	$context =~ s/\./\,/g;  #ldap requires all . to be replaced by , #FIXME --> Is this needed?
	$locatorbase=$context; #set the context base to the current context
	$mesg = $ldap->search (
	# perform a search
			base => $locatorbase, scope=>"base", filter => "(objectclass=*)",attrs=> ['nspmPasswordPolicyDN']);
	my @cxt=$mesg->entries;
	my $numofcontext = $mesg->count;  #if the context is not migrated this count will be 0
	if($numofcontext ne 0)
	{
		if($cxt[0]->get_value("nspmPasswordPolicyDN") eq ""){ #the context is not attached to any pwd policy
			createPasswordPolicyattachingldif($cxt[0]->dn);
		
		}
		else
		{ # the context is attached to some policy..now need to check if it meets our requirements
			if(!checkifpolicyispwdenabled($cxt[0]->get_value("nspmPasswordPolicyDN")))
			{
				#this means that the policy to which the context is currently attached does not exist
				# replace the exisiting policy with our default one
				modifyPasswordpolicyattachingldif($cxt[0]->dn);
			}
		}
	}
	else
	{
		logmsg "Failed to attach password policy to context $_[0]. Probably the context does not exist on the target tree","","Information";
	}
}


sub readsrcfile()
{

	if ( !open (AFP_IN_HANDLE,"<$srcfile") )
 	{
		logmsg "Could not open $srcfile", $!,"Fatal";
		exit 2;
 	}
 	my $newline;

	foreach(<AFP_IN_HANDLE>)
	{
	    	chomp;

		if ( ! /^[#\n\r]/ )
		{
			$newline=$_;
			if (defined ($newline))
			{
		  		$newline =~ s/^\s+//;
		  		$newline =~ s/\s+$//;
			}
			push (@cxts, $newline);
        	}
	}
	close (AFP_IN_HANDLE);
}


%args=();
#getopt("b:x:q:t:g:f",\%args);
#getopt("b:q:t:g:f",\%args);
getopt("b:q:t:f",\%args);

$binddn = $args{b};
$bindpasswd = $ENV{'TARGETPASSWD'};
$port   = $args{q};
$ssl	= $args{t};
#$proxyuser = $args{g};
$source = gethostip();
$pwdpolicyfile = $args{f};

# should replace the . with ,
$proxyuser =~ s/\./\,/g;

logmsg "The proxy credentials which were retrieved are: $proxyuser","","Information";

#need to check how to log here as well
if ( !open (LOGFILE, ">>/var/log/migafp.log") )
{
	exit 2;
}

readsrcfile();

$count=@cxts;
establishconnection();

if($count ne 0)
{
	logmsg "Starting the Universal Password enabling of contexts","","Information";
	#check if the default AFP-CIFS Migration Policy exists.Else create it. 
	if(!checkifdefaultpolicyexists())
	{
 		logmsg "AFP-CIFS Migration Policy does not exist. Creating AFP-CIFS Migration Policy","","Information";
 		createpwdpolicy();
		#attachProxyUsertodefaultPolicy(); #Assigning the proxy user to the default password policy. Just a precaution.Might fail but no issues
	}
	else
	{
 		logmsg "AFP-CIFS Migration Policy password policy exists. Assigning this to AFP user contexts","","Information";
	}

	for my $p (0 .. $#cxts)
	{
       		#universalcheckuser($cxts[$p]);
       		bindcontexttopolicy($cxts[$p]);
	}
}
else
{
	logmsg "No context were migrated from the source server.So no modification to be done","","Information";
}


for (@modifiedPolicies) { $modified_policy_hash{$_} = 1 }
logmsg "The modified policies map is @modifiedPolicies","","Information";
modifyPolicies();
$ldap->unbind;
my $err ;

if($ssl) {
	$err = `env LDAPTLS_CACERT=/etc/opt/novell/certs/SSCert.pem /usr/bin/ldapmodify -a  -H ldaps://"$source":$port -f /tmp/afpuserpwdpolicy.ldif -x -c -w "$bindpasswd" -D "$binddn"`
}
else 
{
	$err = `env LDAPTLS_CACERT=/etc/opt/novell/certs/SSCert.pem /usr/bin/ldapmodify -a  -H ldap://"$source":$port -f /tmp/afpuserpwdpolicy.ldif -x -c -w "$bindpasswd" -D "$binddn" -Z`
}
if($err ne "")
{
	if(($? >> 8) != 20) # only if it is not duplicate value error, fail
	{
		logmsg "An error occurred while trying to load the ldif files on the target tree. Please see the logfile for more details.",$!, "Error";
		logmsg "$err",$!, "Error";
		close (LOGFILE);
        	exit (1);
	}

}
close (LOGFILE);
exit 0;
